@@ -105,6 +105,24 @@
 //! }
 //! ```
 //!
+//! #### Explicit property inheritance
+//!
+//! ```
+//! #[derive(Debug, thiserror::Error, deno_error::JsError)]
+//! #[class(generic)]
+//! #[properties(inherit)]
+//! #[error(transparent)]
+//! pub struct SomeError(std::io::Error);
+//! ```
+//!
+//! ```
+//! #[derive(Debug, thiserror::Error, deno_error::JsError)]
+//! #[class(inherit)]
+//! #[properties(no_inherit)]
+//! #[error(transparent)]
+//! pub struct SomeError(std::io::Error);
+//! ```
+//!
 //! #### Inferred inheritance
 //! ```
 //! #[derive(Debug, thiserror::Error, deno_error::JsError)]
@@ -161,6 +179,73 @@ pub mod builtin_classes {
 }
 use builtin_classes::*;
 
+/// Represents a property value that can be either a string or a number
+#[derive(Debug, Clone, PartialEq)]
+pub enum PropertyValue {
+  String(Cow<'static, str>),
+  Number(f64),
+}
+
+impl std::fmt::Display for PropertyValue {
+  fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    match self {
+      PropertyValue::String(s) => write!(f, "{}", s),
+      PropertyValue::Number(n) => write!(f, "{}", n),
+    }
+  }
+}
+
+impl From<String> for PropertyValue {
+  fn from(s: String) -> Self {
+    PropertyValue::String(Cow::Owned(s))
+  }
+}
+
+impl From<&'static str> for PropertyValue {
+  fn from(s: &'static str) -> Self {
+    PropertyValue::String(Cow::Borrowed(s))
+  }
+}
+
+impl From<f64> for PropertyValue {
+  fn from(n: f64) -> Self {
+    PropertyValue::Number(n)
+  }
+}
+
+impl From<&f64> for PropertyValue {
+  fn from(n: &f64) -> Self {
+    PropertyValue::Number(*n)
+  }
+}
+
+impl From<i32> for PropertyValue {
+  fn from(n: i32) -> Self {
+    PropertyValue::Number(n as f64)
+  }
+}
+
+impl From<&i32> for PropertyValue {
+  fn from(n: &i32) -> Self {
+    PropertyValue::Number(*n as f64)
+  }
+}
+
+impl From<u32> for PropertyValue {
+  fn from(n: u32) -> Self {
+    PropertyValue::Number(n as f64)
+  }
+}
+
+impl From<&u32> for PropertyValue {
+  fn from(n: &u32) -> Self {
+    PropertyValue::Number(*n as f64)
+  }
+}
+
+pub type AdditionalProperties =
+  Box<dyn Iterator<Item = (Cow<'static, str>, PropertyValue)>>;
+
 /// Trait to implement how an error should be represented in JavaScript.
 ///
 /// **Note**:
@@ -176,9 +261,7 @@ pub trait JsErrorClass:
   fn get_message(&self) -> Cow<'static, str>;
 
   /// Additional properties that should be defined on the error in JavaScript side.
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)>;
+  fn get_additional_properties(&self) -> AdditionalProperties;
 
   fn as_any(&self) -> &dyn Any;
 }
@@ -238,13 +321,8 @@ macro_rules! js_error_wrapper {
       fn get_message(&self) -> std::borrow::Cow<'static, str> {
         self.to_string().into()
       }
-      fn get_additional_properties(
-        &self,
-      ) -> Vec<(
-        std::borrow::Cow<'static, str>,
-        std::borrow::Cow<'static, str>,
-      )> {
-        vec![]
+      fn get_additional_properties(&self) -> deno_error::AdditionalProperties {
+        Box::new(std::iter::empty())
       }
       fn as_any(&self) -> &dyn std::any::Any {
         self
@@ -269,9 +347,7 @@ impl<T: JsErrorClass> JsErrorClass for Box<T> {
     (**self).get_message()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
+  fn get_additional_properties(&self) -> AdditionalProperties {
     (**self).get_additional_properties()
   }
 
@@ -303,16 +379,13 @@ impl JsErrorClass for std::io::Error {
       UnexpectedEof => "UnexpectedEof",
       Other => GENERIC_ERROR,
       WouldBlock => "WouldBlock",
-      kind => {
-        let kind_str = kind.to_string();
-        match kind_str.as_str() {
-          "FilesystemLoop" => "FilesystemLoop",
-          "IsADirectory" => "IsADirectory",
-          "NetworkUnreachable" => "NetworkUnreachable",
-          "NotADirectory" => "NotADirectory",
-          _ => GENERIC_ERROR,
-        }
-      }
+      IsADirectory => "IsADirectory",
+      NetworkUnreachable => "NetworkUnreachable",
+      NotADirectory => "NotADirectory",
+      kind => match format!("{kind:?}").as_str() {
+        "FilesystemLoop" => "FilesystemLoop",
+        _ => GENERIC_ERROR,
+      },
     };
 
     Cow::Borrowed(class)
@@ -322,12 +395,15 @@ impl JsErrorClass for std::io::Error {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    get_error_code(self)
-      .map(|code| vec![("code".into(), code.into())])
-      .unwrap_or_default()
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    if let Some(code) = get_error_code(self) {
+      Box::new(std::iter::once((
+        "code".into(),
+        PropertyValue::String(code.into()),
+      )))
+    } else {
+      Box::new(Box::new(std::iter::empty()))
+    }
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -347,10 +423,8 @@ impl JsErrorClass for std::env::VarError {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![]
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty())
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -367,10 +441,8 @@ impl JsErrorClass for std::sync::mpsc::RecvError {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![]
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty())
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -387,10 +459,8 @@ impl JsErrorClass for std::str::Utf8Error {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![]
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty())
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -407,10 +477,8 @@ impl JsErrorClass for std::num::TryFromIntError {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![]
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty())
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -427,10 +495,8 @@ impl JsErrorClass for std::convert::Infallible {
     unreachable!()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    unreachable!()
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    unreachable!();
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -460,10 +526,8 @@ impl JsErrorClass for serde_json::Error {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![] // TODO: could be io error code
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty()) // TODO: could be io error code
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -481,10 +545,8 @@ impl JsErrorClass for url::ParseError {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![]
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty())
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -504,10 +566,8 @@ impl<T: Send + Sync + 'static> JsErrorClass
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![]
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty())
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -525,10 +585,8 @@ impl JsErrorClass for tokio::task::JoinError {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![]
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty())
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -546,10 +604,8 @@ impl JsErrorClass for tokio::sync::broadcast::error::RecvError {
     self.to_string().into()
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
-    vec![]
+  fn get_additional_properties(&self) -> AdditionalProperties {
+    Box::new(std::iter::empty())
   }
 
   fn as_any(&self) -> &dyn Any {
@@ -615,11 +671,9 @@ impl JsErrorClass for JsErrorBox {
     }
   }
 
-  fn get_additional_properties(
-    &self,
-  ) -> Vec<(Cow<'static, str>, Cow<'static, str>)> {
+  fn get_additional_properties(&self) -> AdditionalProperties {
     match &self.0 {
-      JsErrorBoxInner::Standalone { .. } => vec![],
+      JsErrorBoxInner::Standalone { .. } => Box::new(std::iter::empty()),
       JsErrorBoxInner::Wrap(inner) => inner.get_additional_properties(),
     }
   }
@@ -668,3 +722,27 @@ impl JsErrorBox {
     Self::new(NOT_SUPPORTED_ERROR, "The operation is not supported")
   }
 }
+
+#[cfg(test)]
+mod tests {
+  use std::io;
+
+  use super::JsErrorClass;
+
+  #[test]
+  fn test_io_error_class_stable() {
+    assert_eq!(
+      io::Error::new(io::ErrorKind::NotFound, "").get_class(),
+      "NotFound",
+    );
+  }
+
+  #[test]
+  #[cfg(unix)]
+  fn test_io_error_class_unstable() {
+    assert_eq!(
+      io::Error::from_raw_os_error(libc::ELOOP).get_class(),
+      "FilesystemLoop",
+    );
+  }
+}
