@@ -1,5 +1,7 @@
 // Copyright 2018-2024 the Deno authors. MIT license.
 
+use std::borrow::Cow;
+
 use anyhow::Result;
 use anyhow::bail;
 use monch::*;
@@ -248,6 +250,8 @@ pub enum WordPart {
   Text(String),
   /// Variable substitution (ex. `$MY_VAR`)
   Variable(String),
+  /// Tilde expansion (ex. `~`)
+  Tilde,
   /// Command substitution (ex. `$(command)`)
   Command(SequentialList),
   /// Quoted string (ex. `"hello"` or `'test'`)
@@ -691,15 +695,82 @@ fn parse_single_quoted_string(input: &str) -> ParseResult<&str> {
 
 fn parse_double_quoted_string(input: &str) -> ParseResult<Vec<WordPart>> {
   // https://pubs.opengroup.org/onlinepubs/009604499/utilities/xcu_chap02.html#tag_02_02_03
-  // Double quotes may have escaped
-  delimited(
-    ch('"'),
+  parse_surrounded_expression(
+    input,
+    '"',
+    "Expected closing double quote.",
     parse_word_parts(ParseWordPartsMode::DoubleQuotes),
-    with_failure_input(
-      input,
-      assert_exists(ch('"'), "Expected closing double quote."),
-    ),
-  )(input)
+  )
+}
+
+fn parse_surrounded_expression<'a, TResult>(
+  input: &'a str,
+  surrounded_char: char,
+  fail_message: &str,
+  parse: impl Fn(&str) -> ParseResult<TResult>,
+) -> ParseResult<'a, TResult> {
+  let start_input = input;
+  let (input, _) = ch(surrounded_char)(input)?;
+  let mut was_escape = false;
+  for (index, c) in input.char_indices() {
+    match c {
+      c if c == surrounded_char && !was_escape => {
+        let inner_input = &input[..index];
+        let inner_input =
+          if surrounded_char == '`' && inner_input.contains("\\`") {
+            Cow::Owned(inner_input.replace("\\`", "`"))
+          } else {
+            Cow::Borrowed(inner_input)
+          };
+        let parts = match parse(&inner_input) {
+          Ok((result_input, parts)) => {
+            if !result_input.is_empty() {
+              return ParseError::fail(
+                input,
+                format!(
+                  "Failed parsing within {}. Unexpected character: {}",
+                  if c == '`' {
+                    "backticks"
+                  } else {
+                    "double quotes"
+                  },
+                  result_input
+                ),
+              );
+            }
+            parts
+          }
+          Err(err) => {
+            return ParseError::fail(
+              input,
+              format!(
+                "Failed parsing within {}. {}",
+                if c == '`' {
+                  "backticks"
+                } else {
+                  "double quotes"
+                },
+                match &err {
+                  ParseError::Backtrace => "Could not determine expression.",
+                  ParseError::Failure(parse_error_failure) =>
+                    parse_error_failure.message.as_str(),
+                }
+              ),
+            );
+          }
+        };
+        return Ok((&input[index + 1..], parts));
+      }
+      '\\' => {
+        was_escape = true;
+      }
+      _ => {
+        was_escape = false;
+      }
+    }
+  }
+
+  ParseError::fail(start_input, fail_message)
 }
 
 #[derive(Clone, Copy, PartialEq, Eq)]
@@ -751,38 +822,46 @@ fn parse_word_parts(
     or7(
       parse_special_shell_var,
       parse_escaped_dollar_sign,
+      parse_escaped_char('~'),
       parse_escaped_char('`'),
       parse_escaped_char('"'),
       parse_escaped_char('('),
-      parse_escaped_char(')'),
-      if_true(parse_escaped_char('\''), move |_| {
-        mode == ParseWordPartsMode::DoubleQuotes
-      }),
+      or(
+        parse_escaped_char(')'),
+        if_true(parse_escaped_char('\''), move |_| {
+          mode == ParseWordPartsMode::DoubleQuotes
+        }),
+      ),
     )
   }
 
+  fn append_char(result: &mut Vec<WordPart>, c: char) {
+    if let Some(WordPart::Text(text)) = result.last_mut() {
+      text.push(c);
+    } else {
+      result.push(WordPart::Text(c.to_string()));
+    }
+  }
+
   move |input| {
     enum PendingPart<'a> {
       Char(char),
       Variable(&'a str),
+      Tilde,
       Command(SequentialList),
       Parts(Vec<WordPart>),
     }
 
+    let original_input = input;
     let (input, parts) = many0(or7(
-      or(
+      or3(
         map(tag("$?"), |_| PendingPart::Variable("?")),
         map(first_escaped_char(mode), PendingPart::Char),
+        map(parse_command_substitution, PendingPart::Command),
       ),
-      map(parse_command_substitution, PendingPart::Command),
+      map(parse_backticks_command_substitution, PendingPart::Command),
+      map(ch('~'), |_| PendingPart::Tilde),
       map(preceded(ch('$'), parse_env_var_name), PendingPart::Variable),
-      |input| {
-        let (_, _) = ch('`')(input)?;
-        ParseError::fail(
-          input,
-          "Back ticks in strings is currently not supported.",
-        )
-      },
       // words can have escaped spaces
       map(
         if_true(preceded(ch('\\'), ch(' ')), |_| {
@@ -812,22 +891,32 @@ fn parse_word_parts(
     ))(input)?;
 
     let mut result = Vec::new();
-    for part in parts {
+    let mut parts = parts.into_iter().enumerate().peekable();
+    while let Some((i, part)) = parts.next() {
       match part {
         PendingPart::Char(c) => {
-          if let Some(WordPart::Text(text)) = result.last_mut() {
-            text.push(c);
+          append_char(&mut result, c);
+        }
+        PendingPart::Tilde => {
+          if i == 0 {
+            if matches!(parts.peek(), None | Some((_, PendingPart::Char('/'))))
+            {
+              result.push(WordPart::Tilde);
+            } else {
+              return ParseError::fail(
+                original_input,
+                "Unsupported tilde expansion.",
+              );
+            }
           } else {
-            result.push(WordPart::Text(c.to_string()));
+            append_char(&mut result, '~');
           }
         }
         PendingPart::Command(s) => result.push(WordPart::Command(s)),
         PendingPart::Variable(v) => {
-          result.push(WordPart::Variable(v.to_string()))
-        }
-        PendingPart::Parts(parts) => {
-          result.extend(parts);
+          result.push(WordPart::Variable(v.to_string()));
         }
+        PendingPart::Parts(parts) => result.extend(parts),
       }
     }
 
@@ -836,7 +925,28 @@ fn parse_word_parts(
 }
 
 fn parse_command_substitution(input: &str) -> ParseResult<SequentialList> {
-  delimited(tag("$("), parse_sequential_list, ch(')'))(input)
+  delimited(
+    tag("$("),
+    parse_sequential_list,
+    with_failure_input(
+      input,
+      assert_exists(
+        ch(')'),
+        "Expected closing parenthesis for command substitution.",
+      ),
+    ),
+  )(input)
+}
+
+fn parse_backticks_command_substitution(
+  input: &str,
+) -> ParseResult<SequentialList> {
+  parse_surrounded_expression(
+    input,
+    '`',
+    "Expected closing backtick.",
+    parse_sequential_list,
+  )
 }
 
 fn parse_subshell(input: &str) -> ParseResult<SequentialList> {
@@ -950,19 +1060,21 @@ mod test {
       parse("cmd 'test").err().unwrap().to_string(),
       concat!("Expected closing single quote.\n", "  'test\n", "  ~"),
     );
+    assert_eq!(
+      parse("cmd \"test$(echo testing\"")
+        .err()
+        .unwrap()
+        .to_string(),
+      concat!(
+        "Failed parsing within double quotes. Expected closing parenthesis for command substitution.\n",
+        "  test$(echo testing\"\n",
+        "  ~"
+      ),
+    );
 
     assert!(parse("( test ||other&&test;test);(t&est );").is_ok());
     assert!(parse("command --arg='value'").is_ok());
     assert!(parse("command --arg=\"value\"").is_ok());
-
-    assert_eq!(
-      parse("echo `echo 1`").err().unwrap().to_string(),
-      concat!(
-        "Back ticks in strings is currently not supported.\n",
-        "  `echo 1`\n",
-        "  ~",
-      ),
-    );
     assert!(
       parse("deno run --allow-read=. --allow-write=./testing main.ts").is_ok(),
     );
@@ -1416,7 +1528,7 @@ mod test {
     run_test(
       parse_quoted_string,
       r#""asdf`""#,
-      Err("Back ticks in strings is currently not supported."),
+      Err("Failed parsing within double quotes. Expected closing backtick."),
     );
 
     run_test_with_end(
@@ -1427,6 +1539,25 @@ mod test {
     );
   }
 
+  #[test]
+  fn tilde_expansion() {
+    run_test(
+      parse_word_parts(ParseWordPartsMode::Unquoted),
+      r#"~test"#,
+      Err("Unsupported tilde expansion."),
+    );
+    run_test(
+      parse_word_parts(ParseWordPartsMode::Unquoted),
+      r#"~+/test"#,
+      Err("Unsupported tilde expansion."),
+    );
+    run_test(
+      parse_word_parts(ParseWordPartsMode::Unquoted),
+      r#"~/test"#,
+      Ok(vec![WordPart::Tilde, WordPart::Text("/test".to_string())]),
+    );
+  }
+
   #[test]
   fn test_parse_word() {
     run_test(parse_unquoted_word, "if", Err("Unsupported reserved word."));
