@@ -65,7 +65,7 @@ pub fn resolve_command_path<'a>(
       // using the deno as the current executable
       let file_stem = exe_path.file_stem().map(|s| s.to_string_lossy());
       if file_stem
-        .map(|s| !s.starts_with("deno_task_shell-"))
+        .map(|s| !s.starts_with("integration_test-"))
         .unwrap_or(true)
       {
         return Ok(exe_path);
