@@ -250,6 +250,7 @@ impl PackageCache {
         }
         // Sha256 of the expected package
         let sha256 = cache_key.sha256();
+        let md5 = cache_key.md5();
         let download_reporter = reporter.clone();
         // Get or fetch the package, using the specified fetch function
         self.get_or_fetch(cache_key, move |destination| {
@@ -276,8 +277,33 @@ impl PackageCache {
                     )
                         .await;
 
-                    // Extract any potential error
-                    let Err(err) = result else { return Ok(()); };
+                    let err = match result {
+                        Ok(result) => {
+                            if let Some(md5) = md5 {
+                                if md5 != result.md5 {
+                                    // Delete the package if the hash does not match
+                                    tokio_fs::remove_dir_all(&destination).await.unwrap();
+                                    return Err(ExtractError::HashMismatch {
+                                        expected: format!("{md5:x}"),
+                                        actual: format!("{:x}", result.md5),
+                                    });
+                                }
+                            }
+
+                            if let Some(sha256) = sha256 {
+                                if sha256 != result.sha256 {
+                                    // Delete the package if the hash does not match
+                                    tokio_fs::remove_dir_all(&destination).await.unwrap();
+                                    return Err(ExtractError::HashMismatch {
+                                        expected: format!("{sha256:x}"),
+                                        actual: format!("{:x}", result.sha256),
+                                    });
+                                }
+                            }
+                            return Ok(());
+                        }
+                        Err(err) => err,
+                    };
 
                     // Only retry on io errors. We assume that the user has
                     // middleware installed that handles connection retries.
@@ -421,9 +447,11 @@ where
         } else if !cache_dir_exists {
             tracing::debug!("cache directory does not exist, fetching package");
         } else if hash_mismatch {
-            tracing::warn!("hash mismatch, wanted a package with hash {} but the cached package has hash {}, fetching package",
+            tracing::warn!(
+                "hash mismatch, wanted a package with hash {} but the cached package has hash {}, fetching package",
                 given_sha.map_or(String::from("<unknown>"), |s| format!("{s:x}")),
-                locked_sha256.map_or(String::from("<unknown>"), |s| format!("{s:x}")));
+                locked_sha256.map_or(String::from("<unknown>"), |s| format!("{s:x}"))
+            );
         }
 
         // If the cache is stale, we need to fetch the package again. We have to acquire
@@ -824,7 +852,7 @@ mod test {
 
         let file_name = get_file_name_from_path(cache_lock_without_origin_hash.path());
         let path_hash = compute_bytes_digest::<Sha256>(package_path.to_string_lossy().as_bytes());
-        let expected_file_name = format!("clobber-python-0.1.0-cpython-{:x}", path_hash);
+        let expected_file_name = format!("clobber-python-0.1.0-cpython-{path_hash:x}");
         assert_eq!(file_name, expected_file_name);
     }
 
