@@ -19,6 +19,7 @@ mod utils;
 mod version;
 pub mod version_spec;
 
+pub mod compression_level;
 mod environment_yaml;
 mod generic_virtual_package;
 pub mod package;
@@ -41,7 +42,7 @@ pub use generic_virtual_package::GenericVirtualPackage;
 pub use match_spec::{
     matcher::{StringMatcher, StringMatcherParseError},
     parse::ParseMatchSpecError,
-    MatchSpec, Matches, NamelessMatchSpec,
+    MatchSpec, MatchSpecUrlError, Matches, NamelessMatchSpec,
 };
 pub use no_arch_type::{NoArchKind, NoArchType, RawNoArchType};
 pub use package_name::{InvalidPackageNameError, PackageName};
