@@ -170,6 +170,7 @@ pub struct PackageRecord {
     /// filled in. So later it can be one of the following:
     /// [`Some(vec![])`] means that the purl is empty and package is not pypi
     /// one. [`Some([`PackageUrl`])`] means that it is a pypi package.
+    /// See this CEP: <https://github.com/conda/ceps/pull/63>
     #[serde(default, skip_serializing_if = "Option::is_none")]
     pub purls: Option<BTreeSet<PackageUrl>>,
 
@@ -515,7 +516,7 @@ impl PackageRecord {
             timestamp: index.timestamp,
             track_features: index.track_features,
             version: index.version,
-            purls: None,
+            purls: index.purls,
             run_exports: None,
         })
     }
