@@ -28,6 +28,9 @@ pub enum ExtractError {
     #[error("an io error occurred: {0}")]
     IoError(#[from] std::io::Error),
 
+    #[error("hash mismatch: expected {expected}, got {actual}")]
+    HashMismatch { expected: String, actual: String },
+
     #[error("could not create the destination path: {0}")]
     CouldNotCreateDestination(#[source] std::io::Error),
 
