@@ -3,18 +3,12 @@
 
 #![allow(clippy::mem_forget)]
 
-use std::{
-    collections::{HashSet, VecDeque},
-    fmt, io,
-    marker::PhantomData,
-    path::Path,
-};
-
 use bytes::Bytes;
 use fs_err as fs;
 use itertools::Itertools;
 use rattler_conda_types::{
-    compute_package_url, Channel, ChannelInfo, PackageName, PackageRecord, RepoDataRecord,
+    compute_package_url, package::ArchiveType, Channel, ChannelInfo, MatchSpec, Matches,
+    PackageName, PackageRecord, RepoDataRecord,
 };
 use rattler_redaction::Redact;
 use serde::{
@@ -22,9 +16,47 @@ use serde::{
     Deserialize, Deserializer,
 };
 use serde_json::value::RawValue;
+use std::borrow::Borrow;
+use std::{
+    collections::{HashSet, VecDeque},
+    fmt, io,
+    marker::PhantomData,
+    path::Path,
+};
 use superslice::Ext;
 use thiserror::Error;
 
+/// Defines how different variants of packages are consolidated.
+#[derive(
+    Default,
+    Debug,
+    Clone,
+    Copy,
+    PartialEq,
+    Eq,
+    strum::Display,
+    strum::VariantNames,
+    strum::EnumString,
+    strum::IntoStaticStr,
+)]
+#[strum(serialize_all = "kebab-case")]
+pub enum PackageFormatSelection {
+    /// Only the tar.bz2 packages are used
+    OnlyTarBz2,
+
+    /// Only the conda packages are used
+    OnlyConda,
+
+    /// Both .tar.bz2 and .conda packages are used, but if a .conda exists that
+    /// represents the same content as a .tar.bz2, the .conda package is
+    /// selected and the .tar.bz2 is discarded.
+    #[default]
+    PreferConda,
+
+    /// Both .tar.bz2 and .conda packages are used
+    Both,
+}
+
 /// A struct to enable loading records from a `repodata.json` file on demand.
 /// Since most of the time you don't need all the records from the
 /// `repodata.json` this can help provide some significant speedups.
@@ -165,38 +197,128 @@ impl SparseRepoData {
     /// This works by iterating over all elements in the `packages` and
     /// `conda_packages` fields of the repodata and returning the unique
     /// package names.
-    pub fn package_names(&self) -> impl Iterator<Item = &'_ str> + '_ {
+    pub fn package_names(
+        &self,
+        package_format_selection: PackageFormatSelection,
+    ) -> impl Iterator<Item = &'_ str> {
+        fn select_package_name<'i>((filename, _): &(PackageFilename<'i>, &'i RawValue)) -> &'i str {
+            filename.package
+        }
+
         let repo_data = self.inner.borrow_repo_data();
-        repo_data
-            .packages
-            .iter()
-            .chain(repo_data.conda_packages.iter())
-            .map(|(name, _)| name.package)
-            .dedup()
+        let tar_baz2_packages = repo_data.packages.iter().map(select_package_name);
+        let conda_packages = repo_data.conda_packages.iter().map(select_package_name);
+
+        match package_format_selection {
+            PackageFormatSelection::Both | PackageFormatSelection::PreferConda => {
+                itertools::Either::Left(tar_baz2_packages.merge(conda_packages).dedup())
+            }
+            PackageFormatSelection::OnlyTarBz2 => {
+                itertools::Either::Right(tar_baz2_packages.dedup())
+            }
+            PackageFormatSelection::OnlyConda => itertools::Either::Right(conda_packages.dedup()),
+        }
+    }
+
+    /// Returns the number of records in this instance.
+    pub fn record_count(&self, package_format_selection: PackageFormatSelection) -> usize {
+        match package_format_selection {
+            PackageFormatSelection::PreferConda => {
+                let repo_data = self.inner.borrow_repo_data();
+                let tar_bz2_packages = repo_data.packages.iter().map(|(filename, _)| {
+                    filename
+                        .filename
+                        .strip_suffix(ArchiveType::TarBz2.extension())
+                        .unwrap_or(filename.filename)
+                });
+                let conda_packages = repo_data.conda_packages.iter().map(|(filename, _)| {
+                    filename
+                        .filename
+                        .strip_suffix(ArchiveType::Conda.extension())
+                        .unwrap_or(filename.filename)
+                });
+                conda_packages.merge(tar_bz2_packages).dedup().count()
+            }
+            PackageFormatSelection::Both => {
+                self.inner.borrow_repo_data().packages.len()
+                    + self.inner.borrow_repo_data().conda_packages.len()
+            }
+            PackageFormatSelection::OnlyTarBz2 => self.inner.borrow_repo_data().packages.len(),
+            PackageFormatSelection::OnlyConda => self.inner.borrow_repo_data().conda_packages.len(),
+        }
+    }
+
+    /// Returns all the records that matches any of the specified match spec.
+    pub fn load_matching_records(
+        &self,
+        spec: impl IntoIterator<Item = impl Borrow<MatchSpec>>,
+        variant_consolidation: PackageFormatSelection,
+    ) -> io::Result<Vec<RepoDataRecord>> {
+        let mut result = Vec::new();
+        let repo_data = self.inner.borrow_repo_data();
+        let base_url = repo_data.info.as_ref().and_then(|i| i.base_url.as_deref());
+        for (package_name, specs) in &spec.into_iter().chunk_by(|spec| spec.borrow().name.clone()) {
+            let grouped_specs = specs.into_iter().collect::<Vec<_>>();
+            let mut parsed_records = parse_records(
+                package_name.as_ref(),
+                &repo_data.packages,
+                &repo_data.conda_packages,
+                variant_consolidation,
+                base_url,
+                &self.channel,
+                &self.subdir,
+                self.patch_record_fn,
+                |record| {
+                    grouped_specs
+                        .iter()
+                        .any(|spec| spec.borrow().matches(&record.package_record))
+                },
+            )?;
+            result.append(&mut parsed_records);
+        }
+
+        Ok(result)
     }
 
     /// Returns all the records for the specified package name.
-    pub fn load_records(&self, package_name: &PackageName) -> io::Result<Vec<RepoDataRecord>> {
+    pub fn load_records(
+        &self,
+        package_name: &PackageName,
+        variant_consolidation: PackageFormatSelection,
+    ) -> io::Result<Vec<RepoDataRecord>> {
         let repo_data = self.inner.borrow_repo_data();
         let base_url = repo_data.info.as_ref().and_then(|i| i.base_url.as_deref());
-        let mut records = parse_records(
-            package_name,
+        parse_records(
+            Some(package_name),
             &repo_data.packages,
-            base_url,
-            &self.channel,
-            &self.subdir,
-            self.patch_record_fn,
-        )?;
-        let mut conda_records = parse_records(
-            package_name,
             &repo_data.conda_packages,
+            variant_consolidation,
             base_url,
             &self.channel,
             &self.subdir,
             self.patch_record_fn,
-        )?;
-        records.append(&mut conda_records);
-        Ok(records)
+            |_| true, // Dont filter anything out
+        )
+    }
+
+    /// Returns all the records for the specified package format(s).
+    pub fn load_all_records(
+        &self,
+        variant_consolidation: PackageFormatSelection,
+    ) -> io::Result<Vec<RepoDataRecord>> {
+        let repo_data = self.inner.borrow_repo_data();
+        let base_url = repo_data.info.as_ref().and_then(|i| i.base_url.as_deref());
+        parse_records(
+            None,
+            &repo_data.packages,
+            &repo_data.conda_packages,
+            variant_consolidation,
+            base_url,
+            &self.channel,
+            &self.subdir,
+            self.patch_record_fn,
+            |_| true,
+        )
     }
 
     /// Given a set of [`SparseRepoData`]s load all the records for the packages
@@ -209,6 +331,7 @@ impl SparseRepoData {
         repo_data: impl IntoIterator<Item = &'a SparseRepoData>,
         package_names: impl IntoIterator<Item = PackageName>,
         patch_function: Option<fn(&mut PackageRecord)>,
+        variant_consolidation: PackageFormatSelection,
     ) -> io::Result<Vec<Vec<RepoDataRecord>>> {
         let repo_data: Vec<_> = repo_data.into_iter().collect();
 
@@ -233,22 +356,16 @@ impl SparseRepoData {
 
                 // Get all records from the repodata
                 let mut records = parse_records(
-                    &next_package,
+                    Some(&next_package),
                     &repo_data_packages.packages,
-                    base_url,
-                    &repo_data.channel,
-                    &repo_data.subdir,
-                    patch_function,
-                )?;
-                let mut conda_records = parse_records(
-                    &next_package,
                     &repo_data_packages.conda_packages,
+                    variant_consolidation,
                     base_url,
                     &repo_data.channel,
                     &repo_data.subdir,
                     patch_function,
+                    |_| true,
                 )?;
-                records.append(&mut conda_records);
 
                 // Iterate over all packages to find recursive dependencies.
                 for record in records.iter() {
@@ -302,50 +419,179 @@ struct LazyRepoData<'i> {
     conda_packages: Vec<(PackageFilename<'i>, &'i RawValue)>,
 }
 
+/// Returns an iterator over the packages in the slice that match the given
+/// package name.
+fn find_package_in_slice<'a, 'i: 'a>(
+    slice: &'a [(PackageFilename<'i>, &'i RawValue)],
+    package_name: Option<&PackageName>,
+) -> impl Iterator<Item = (PackageFilename<'i>, &'i RawValue)> + 'a {
+    let range = match package_name {
+        None => 0..slice.len(),
+        Some(package_name) => {
+            slice.equal_range_by(|(package, _)| package.package.cmp(package_name.as_normalized()))
+        }
+    };
+
+    slice[range]
+        .iter()
+        .map(|(filename, raw_json)| (*filename, *raw_json))
+}
+
+/// Takes an iterator over package filenames and raw json values and returns an
+/// iterator that also includes the filename without an extension.
+fn add_stripped_filename<'i>(
+    slice: impl Iterator<Item = (PackageFilename<'i>, &'i RawValue)>,
+    ext: ArchiveType,
+) -> impl Iterator<Item = (PackageFilename<'i>, &'i RawValue, &'i str)> {
+    slice.map(move |(filename, raw_json)| {
+        (
+            filename,
+            raw_json,
+            filename
+                .filename
+                .strip_suffix(ext.extension())
+                .unwrap_or(filename.filename),
+        )
+    })
+}
+
 /// Parse the records for the specified package from the raw index
-fn parse_records<'i>(
-    package_name: &PackageName,
-    packages: &[(PackageFilename<'i>, &'i RawValue)],
+#[allow(clippy::too_many_arguments)]
+fn parse_records<'i, F: Fn(&RepoDataRecord) -> bool>(
+    package_name: Option<&PackageName>,
+    tar_bz2_packages: &[(PackageFilename<'i>, &'i RawValue)],
+    conda_packages: &[(PackageFilename<'i>, &'i RawValue)],
+    variant_consolidation: PackageFormatSelection,
     base_url: Option<&str>,
     channel: &Channel,
     subdir: &str,
     patch_function: Option<fn(&mut PackageRecord)>,
+    filter_function: F,
 ) -> io::Result<Vec<RepoDataRecord>> {
-    let channel_name = channel.base_url.clone();
-
-    let package_indices =
-        packages.equal_range_by(|(package, _)| package.package.cmp(package_name.as_normalized()));
-    let mut result = Vec::with_capacity(package_indices.len());
-    for (key, raw_json) in &packages[package_indices] {
-        let mut package_record: PackageRecord = serde_json::from_str(raw_json.get())?;
-        // Overwrite subdir if its empty
-        if package_record.subdir.is_empty() {
-            package_record.subdir = subdir.to_owned();
-        }
-        result.push(RepoDataRecord {
-            url: compute_package_url(
-                &channel
-                    .base_url
-                    .url()
-                    .join(&format!("{}/", &package_record.subdir))
-                    .expect("failed determine repo_base_url"),
+    match variant_consolidation {
+        PackageFormatSelection::PreferConda => {
+            let tar_bz2_packages = add_stripped_filename(
+                find_package_in_slice(tar_bz2_packages, package_name),
+                ArchiveType::TarBz2,
+            );
+            let conda_packages = add_stripped_filename(
+                find_package_in_slice(conda_packages, package_name),
+                ArchiveType::Conda,
+            );
+            let deduplicated_packages = conda_packages
+                // Merge the conda and tar.bz2 packages together based on their filename without
+                // extension.
+                .merge_by(tar_bz2_packages, |(_, _, left), (_, _, right)| {
+                    left <= right
+                })
+                // Deduplicate repeated packages based on their filename without extension. (this
+                // removes the .tar.bz2 in favor of the .conda)
+                .dedup_by(|(_, _, left), (_, _, right)| left == right)
+                .map(|(filename, raw_json, _)| (filename, raw_json));
+            parse_records_raw(
+                deduplicated_packages,
                 base_url,
-                key.filename,
-            ),
-            channel: Some(channel_name.url().clone().redact().to_string()),
-            package_record,
-            file_name: key.filename.to_owned(),
-        });
+                channel,
+                subdir,
+                patch_function,
+                filter_function,
+            )
+        }
+        PackageFormatSelection::Both => {
+            let tar_bz2_packages = find_package_in_slice(tar_bz2_packages, package_name);
+            let conda_packages = find_package_in_slice(conda_packages, package_name);
+            parse_records_raw(
+                tar_bz2_packages.chain(conda_packages),
+                base_url,
+                channel,
+                subdir,
+                patch_function,
+                filter_function,
+            )
+        }
+        PackageFormatSelection::OnlyTarBz2 => {
+            let tar_bz2_packages = find_package_in_slice(tar_bz2_packages, package_name);
+            parse_records_raw(
+                tar_bz2_packages,
+                base_url,
+                channel,
+                subdir,
+                patch_function,
+                filter_function,
+            )
+        }
+        PackageFormatSelection::OnlyConda => {
+            let conda_packages = find_package_in_slice(conda_packages, package_name);
+            parse_records_raw(
+                conda_packages,
+                base_url,
+                channel,
+                subdir,
+                patch_function,
+                filter_function,
+            )
+        }
     }
+}
+
+fn parse_record_raw<'i>(
+    (filename, raw_json): (PackageFilename<'i>, &'i RawValue),
+    base_url: Option<&str>,
+    channel: &Channel,
+    channel_name: Option<String>,
+    subdir: &str,
+    patch_function: Option<fn(&mut PackageRecord)>,
+) -> io::Result<RepoDataRecord> {
+    let mut package_record: PackageRecord = serde_json::from_str(raw_json.get())?;
+    // Overwrite subdir if its empty
+    if package_record.subdir.is_empty() {
+        package_record.subdir = subdir.to_owned();
+    }
+    let mut record = RepoDataRecord {
+        url: compute_package_url(
+            &channel
+                .base_url
+                .url()
+                .join(&format!("{}/", &package_record.subdir))
+                .expect("failed determine repo_base_url"),
+            base_url,
+            filename.filename,
+        ),
+        channel: channel_name.clone(),
+        package_record,
+        file_name: filename.filename.to_owned(),
+    };
 
     // Apply the patch function if one was specified
     if let Some(patch_fn) = patch_function {
-        for record in &mut result {
-            patch_fn(&mut record.package_record);
-        }
+        patch_fn(&mut record.package_record);
     }
 
-    Ok(result)
+    Ok(record)
+}
+
+fn parse_records_raw<'i, F: Fn(&RepoDataRecord) -> bool>(
+    packages: impl Iterator<Item = (PackageFilename<'i>, &'i RawValue)>,
+    base_url: Option<&str>,
+    channel: &Channel,
+    subdir: &str,
+    patch_function: Option<fn(&mut PackageRecord)>,
+    filter_function: F,
+) -> io::Result<Vec<RepoDataRecord>> {
+    let channel_name = channel.base_url.url().clone().redact().to_string();
+    packages
+        .map(move |record| {
+            parse_record_raw(
+                record,
+                base_url,
+                channel,
+                Some(channel_name.clone()),
+                subdir,
+                patch_function,
+            )
+        })
+        .filter_ok(filter_function)
+        .collect()
 }
 
 /// A helper function that immediately loads the records for the given packages
@@ -357,6 +603,7 @@ pub async fn load_repo_data_recursively(
     repo_data_paths: impl IntoIterator<Item = (Channel, impl Into<String>, impl AsRef<Path>)>,
     package_names: impl IntoIterator<Item = PackageName>,
     patch_function: Option<fn(&mut PackageRecord)>,
+    variant_consolidation: PackageFormatSelection,
 ) -> Result<Vec<Vec<RepoDataRecord>>, io::Error> {
     use futures::{StreamExt, TryFutureExt, TryStreamExt};
 
@@ -378,7 +625,12 @@ pub async fn load_repo_data_recursively(
         .try_collect::<Vec<_>>()
         .await?;
 
-    SparseRepoData::load_records_recursive(&lazy_repo_data, package_names, patch_function)
+    SparseRepoData::load_records_recursive(
+        &lazy_repo_data,
+        package_names,
+        patch_function,
+        variant_consolidation,
+    )
 }
 
 fn deserialize_filename_and_raw_record<'d, D: Deserializer<'d>>(
@@ -454,6 +706,7 @@ fn deserialize_filename_and_raw_record<'d, D: Deserializer<'d>>(
 
 /// A struct that holds both a filename and the part of the filename thats just
 /// the package name.
+#[derive(Copy, Clone)]
 struct PackageFilename<'i> {
     package: &'i str,
     filename: &'i str,
@@ -495,15 +748,22 @@ impl<'de> TryFrom<&'de str> for PackageFilename<'de> {
 
 #[cfg(test)]
 mod test {
-    use std::path::{Path, PathBuf};
+    use std::{
+        collections::HashSet,
+        path::{Path, PathBuf},
+    };
 
     use bytes::Bytes;
     use fs_err as fs;
     use itertools::Itertools;
-    use rattler_conda_types::{Channel, ChannelConfig, PackageName, RepoData, RepoDataRecord};
+    use rattler_conda_types::{
+        Channel, ChannelConfig, MatchSpec, PackageName, ParseStrictness, RepoData, RepoDataRecord,
+    };
     use rstest::rstest;
 
-    use super::{load_repo_data_recursively, PackageFilename, SparseRepoData};
+    use super::{
+        load_repo_data_recursively, PackageFilename, PackageFormatSelection, SparseRepoData,
+    };
     use crate::utils::test::fetch_repo_data;
 
     fn test_dir() -> PathBuf {
@@ -528,6 +788,15 @@ mod test {
         ]
     }
 
+    fn dummy_repo_data() -> (Channel, &'static str, PathBuf) {
+        let channel_config = ChannelConfig::default_with_root_dir(std::env::current_dir().unwrap());
+        (
+            Channel::from_str("dummy", &channel_config).unwrap(),
+            "linux-64",
+            test_dir().join("channels/dummy/linux-64/repodata.json"),
+        )
+    }
+
     async fn default_repo_data_bytes() -> Vec<(Channel, &'static str, Bytes)> {
         default_repo_data()
             .await
@@ -542,6 +811,7 @@ mod test {
     fn load_sparse_from_bytes(
         repo_data: &[(Channel, &'static str, Bytes)],
         package_names: impl IntoIterator<Item = impl AsRef<str>>,
+        variant_consolidation: PackageFormatSelection,
     ) -> Vec<Vec<RepoDataRecord>> {
         let sparse: Vec<_> = repo_data
             .iter()
@@ -553,11 +823,13 @@ mod test {
         let package_names = package_names
             .into_iter()
             .map(|name| PackageName::try_from(name.as_ref()).unwrap());
-        SparseRepoData::load_records_recursive(&sparse, package_names, None).unwrap()
+        SparseRepoData::load_records_recursive(&sparse, package_names, None, variant_consolidation)
+            .unwrap()
     }
 
     async fn load_sparse(
         package_names: impl IntoIterator<Item = impl AsRef<str>>,
+        variant_consolidation: PackageFormatSelection,
     ) -> Vec<Vec<RepoDataRecord>> {
         tokio::try_join!(fetch_repo_data("noarch"), fetch_repo_data("linux-64")).unwrap();
 
@@ -570,6 +842,7 @@ mod test {
                 .into_iter()
                 .map(|name| PackageName::try_from(name.as_ref()).unwrap()),
             None,
+            variant_consolidation,
         )
         .await
         .unwrap()
@@ -577,13 +850,15 @@ mod test {
 
     #[tokio::test]
     async fn test_empty_sparse_load() {
-        let sparse_empty_data = load_sparse(Vec::<String>::new()).await;
+        let sparse_empty_data =
+            load_sparse(Vec::<String>::new(), PackageFormatSelection::default()).await;
         assert_eq!(sparse_empty_data, vec![vec![], vec![]]);
     }
 
     #[tokio::test]
     async fn test_sparse_single() {
-        let sparse_empty_data = load_sparse(["_libgcc_mutex"]).await;
+        let sparse_empty_data =
+            load_sparse(["_libgcc_mutex"], PackageFormatSelection::default()).await;
         let total_records = sparse_empty_data
             .iter()
             .map(std::vec::Vec::len)
@@ -594,7 +869,11 @@ mod test {
 
     #[tokio::test]
     async fn test_parse_duplicate() {
-        let sparse_empty_data = load_sparse(["_libgcc_mutex", "_libgcc_mutex"]).await;
+        let sparse_empty_data = load_sparse(
+            ["_libgcc_mutex", "_libgcc_mutex"],
+            PackageFormatSelection::default(),
+        )
+        .await;
         let total_records = sparse_empty_data
             .iter()
             .map(std::vec::Vec::len)
@@ -607,7 +886,11 @@ mod test {
 
     #[tokio::test]
     async fn test_sparse_jupyterlab_detectron2() {
-        let sparse_empty_data = load_sparse(["jupyterlab", "detectron2"]).await;
+        let sparse_empty_data = load_sparse(
+            ["jupyterlab", "detectron2"],
+            PackageFormatSelection::default(),
+        )
+        .await;
 
         let total_records = sparse_empty_data
             .iter()
@@ -619,7 +902,7 @@ mod test {
 
     #[tokio::test]
     async fn test_sparse_rubin_env() {
-        let sparse_empty_data = load_sparse(["rubin-env"]).await;
+        let sparse_empty_data = load_sparse(["rubin-env"], PackageFormatSelection::default()).await;
 
         let total_records = sparse_empty_data
             .iter()
@@ -657,7 +940,8 @@ mod test {
         ];
 
         // Memmapped
-        let sparse_empty_data = load_sparse(package_names.clone()).await;
+        let sparse_empty_data =
+            load_sparse(package_names.clone(), PackageFormatSelection::default()).await;
 
         let total_records = sparse_empty_data.iter().map(Vec::len).sum::<usize>();
 
@@ -665,7 +949,8 @@ mod test {
 
         // Bytes
         let repo_data = default_repo_data_bytes().await;
-        let sparse_empty_data = load_sparse_from_bytes(&repo_data, package_names);
+        let sparse_empty_data =
+            load_sparse_from_bytes(&repo_data, package_names, PackageFormatSelection::default());
 
         let total_records = sparse_empty_data.iter().map(Vec::len).sum::<usize>();
 
@@ -718,6 +1003,99 @@ mod test {
         .unwrap();
 
         assert_eq!(repo_data.packages.len(), 0);
-        assert_eq!(sparse_repodata.package_names().try_len().unwrap(), 0);
+        assert_eq!(
+            sparse_repodata
+                .package_names(PackageFormatSelection::default())
+                .try_len()
+                .unwrap(),
+            0
+        );
+    }
+
+    #[rstest]
+    #[case::both(PackageFormatSelection::Both)]
+    #[case::prefer_conda(PackageFormatSelection::PreferConda)]
+    #[case::only_tar_bz2(PackageFormatSelection::OnlyTarBz2)]
+    #[case::only_conda(PackageFormatSelection::OnlyConda)]
+    fn dedup_packages(#[case] variant: PackageFormatSelection) {
+        let (channel, platform, path) = dummy_repo_data();
+        let sparse = SparseRepoData::from_file(channel, platform, path, None).unwrap();
+        let names = sparse.package_names(variant).collect_vec();
+        let deduped_names = names.iter().copied().collect::<HashSet<_>>();
+        assert_eq!(names.len(), deduped_names.len());
+    }
+
+    #[rstest]
+    #[case::both(PackageFormatSelection::Both)]
+    #[case::prefer_conda(PackageFormatSelection::PreferConda)]
+    #[case::only_tar_bz2(PackageFormatSelection::OnlyTarBz2)]
+    #[case::only_conda(PackageFormatSelection::OnlyConda)]
+    fn test_package_format_selection(#[case] variant: PackageFormatSelection) {
+        let (channel, platform, path) = dummy_repo_data();
+        let sparse = SparseRepoData::from_file(channel, platform, path, None).unwrap();
+        let records = sparse
+            .load_records(&PackageName::try_from("bors").unwrap(), variant)
+            .unwrap()
+            .into_iter()
+            .map(|record| record.file_name)
+            .collect::<Vec<_>>();
+
+        insta::with_settings!({snapshot_suffix => variant.to_string()}, {
+            insta::assert_snapshot!(records.join("\n"));
+        });
+    }
+
+    #[rstest]
+    #[case::both(PackageFormatSelection::Both, 29)]
+    #[case::prefer_conda(PackageFormatSelection::PreferConda, 25)]
+    #[case::only_tar_bz2(PackageFormatSelection::OnlyTarBz2, 24)]
+    #[case::only_conda(PackageFormatSelection::OnlyConda, 5)]
+    fn test_record_count(#[case] variant: PackageFormatSelection, #[case] expected_count: usize) {
+        let (channel, platform, path) = dummy_repo_data();
+        let sparse = SparseRepoData::from_file(channel, platform, path, None).unwrap();
+        let count = sparse.record_count(variant);
+        assert_eq!(count, expected_count);
+    }
+
+    #[test]
+    fn test_query() {
+        let (channel, platform, path) = dummy_repo_data();
+        let sparse = SparseRepoData::from_file(channel, platform, path, None).unwrap();
+        let records = sparse
+            .load_matching_records(
+                vec![
+                    MatchSpec::from_str("bors 1.*", ParseStrictness::Lenient).unwrap(),
+                    MatchSpec::from_str("issue_717", ParseStrictness::Lenient).unwrap(),
+                ],
+                PackageFormatSelection::default(),
+            )
+            .unwrap()
+            .into_iter()
+            .map(|record| record.file_name)
+            .collect::<Vec<_>>();
+
+        insta::assert_snapshot!(records.join("\n"), @r###"
+        bors-1.0-bla_1.tar.bz2
+        bors-1.1-bla_1.conda
+        bors-1.2.1-bla_1.tar.bz2
+        issue_717-2.1-bla_1.conda
+        "###);
+    }
+
+    #[test]
+    fn test_nameless_query() {
+        let (channel, platform, path) = dummy_repo_data();
+        let sparse = SparseRepoData::from_file(channel, platform, path, None).unwrap();
+        let records = sparse
+            .load_matching_records(
+                vec![MatchSpec::from_str("* 12.5", ParseStrictness::Lenient).unwrap()],
+                PackageFormatSelection::default(),
+            )
+            .unwrap()
+            .into_iter()
+            .map(|record| record.file_name)
+            .collect::<Vec<_>>();
+
+        insta::assert_snapshot!(records.join("\n"), @"cuda-version-12.5-hd4f0392_3.conda");
     }
 }
