@@ -8,15 +8,15 @@ use std::{
 use elsa::FrozenMap;
 
 use crate::{
+    Interner, NameId, Requirement,
     internal::{
         arena::{Arena, ArenaId},
         id::{ClauseId, LearntClauseId, StringId, VersionSetId},
     },
     solver::{
-        decision_map::DecisionMap, decision_tracker::DecisionTracker, variable_map::VariableMap,
-        VariableId,
+        VariableId, decision_map::DecisionMap, decision_tracker::DecisionTracker,
+        variable_map::VariableMap,
     },
-    Interner, NameId, Requirement,
 };
 
 /// Represents a single clause in the SAT problem
@@ -556,7 +556,7 @@ pub(crate) struct ClauseDisplay<'i, I: Interner> {
     variable_map: &'i VariableMap,
 }
 
-impl<'i, I: Interner> Display for ClauseDisplay<'i, I> {
+impl<I: Interner> Display for ClauseDisplay<'_, I> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match self.kind {
             Clause::InstallRoot => write!(f, "InstallRoot"),
@@ -632,7 +632,7 @@ mod test {
 
     #[test]
     fn test_literal_eval() {
-        let mut decision_map = DecisionMap::new();
+        let mut decision_map = DecisionMap::default();
 
         let literal = VariableId::root().positive();
         let negated_literal = VariableId::root().negative();
@@ -653,7 +653,7 @@ mod test {
 
     #[test]
     fn test_requires_with_and_without_conflict() {
-        let mut decisions = DecisionTracker::new();
+        let mut decisions = DecisionTracker::default();
 
         let parent = VariableId::from_usize(1);
         let candidate1 = VariableId::from_usize(2);
@@ -740,7 +740,7 @@ mod test {
 
     #[test]
     fn test_constrains_with_and_without_conflict() {
-        let mut decisions = DecisionTracker::new();
+        let mut decisions = DecisionTracker::default();
 
         let parent = VariableId::from_usize(1);
         let forbidden = VariableId::from_usize(2);
